package be.iminds.ilabt.jfed.experiment.setup.config;

import com.fasterxml.jackson.annotation.*;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;

@JsonInclude(JsonInclude.Include.NON_DEFAULT)
@JsonIgnoreProperties(value={"@context"})
public class Slice {
    @Nullable
    private final String sliceName;
    private final boolean failOnExistingSlice;
    private final boolean failOnNonExistingSlice;
    private final long expireTimeMin;
    private final boolean renewExistingSliceIfNeeded;

    @Nonnull
    private final ProjectSource projectSource;
    private final boolean failIfNoProject; //if projectSource is AUTO or PROVIDED
    @Nullable private final String project; //if projectSource is PROVIDED

    public Slice() {
        this.sliceName = null;
        this.failOnExistingSlice = true;
        this.failOnNonExistingSlice = false;
        this.expireTimeMin = 60;
        this.renewExistingSliceIfNeeded = true;
        this.projectSource = ProjectSource.AUTO;
        this.failIfNoProject = false;
        this.project = null;
    }

    @JsonCreator
    public Slice(@Nullable @JsonProperty("sliceName") String sliceName,
                 @Nullable @JsonProperty("failOnExistingSlice") Boolean failOnExistingSlice,
                 @Nullable @JsonProperty("failOnNonExistingSlice") Boolean failOnNonExistingSlice,
                 @Nullable @JsonProperty("expireTimeMin") Long expireTimeMin,
                 @Nullable @JsonProperty("renewExistingSliceIfNeeded") Boolean renewExistingSliceIfNeeded,
                 @Nullable @JsonProperty("projectSource") ProjectSource projectSource,
                 @Nullable @JsonProperty("failIfNoProject") Boolean failIfNoProject,
                 @Nullable @JsonProperty("project") String project) {
        this.sliceName = sliceName == null ? new Slice().getSliceName() : sliceName;
        this.failOnExistingSlice = failOnExistingSlice == null ? new Slice().getFailOnExistingSlice() : failOnExistingSlice;
        this.failOnNonExistingSlice = failOnNonExistingSlice == null ? new Slice().getFailOnNonExistingSlice() : failOnNonExistingSlice;
        this.expireTimeMin = expireTimeMin == null ? new Slice().getExpireTimeMin() : expireTimeMin;
        this.renewExistingSliceIfNeeded = renewExistingSliceIfNeeded == null ? new Slice().getRenewExistingSliceIfNeeded() : renewExistingSliceIfNeeded;
        this.projectSource = projectSource == null ? new Slice().getProjectSource() : projectSource;
        this.failIfNoProject = failIfNoProject == null ? new Slice().getFailIfNoProject() : failIfNoProject;
        this.project = project == null ? new Slice().getProject() : project;
    }

    @JsonIgnore
    public boolean isValid() {
        if (expireTimeMin <= 0)
            throw new IllegalArgumentException("Config Error: slice.expireTimeMin <= makes no sense: "+expireTimeMin);
        if (projectSource == ProjectSource.PROVIDED && project == null && failIfNoProject)
            throw new IllegalArgumentException("Config Error: Combination of failIfNoProject=true and project==null and projectSource == ProjectSource.PROVIDED will always fail");
        if (projectSource == ProjectSource.NONE && failIfNoProject)
            throw new IllegalArgumentException("Config Error: Combination of failIfNoProject=true and projectSource == ProjectSource.NONE will always fail");
        if (failOnExistingSlice && failOnNonExistingSlice)
            throw new IllegalArgumentException("Config Error: failOnExistingSlice and failOnNonExistingSlice should not both be true");

        return true;
    }

    @Nullable
    public String getSliceName() {
        return sliceName;
    }

    public boolean getFailOnExistingSlice() {
        return failOnExistingSlice;
    }

    public boolean getFailOnNonExistingSlice() {
        return failOnNonExistingSlice;
    }

    public long getExpireTimeMin() {
        return expireTimeMin;
    }

    public boolean getRenewExistingSliceIfNeeded() {
        return renewExistingSliceIfNeeded;
    }

    @Nonnull
    public ProjectSource getProjectSource() {
        return projectSource;
    }

    public boolean getFailIfNoProject() {
        return failIfNoProject;
    }

    @Nullable
    public String getProject() {
        return project;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof Slice)) return false;

        Slice slice = (Slice) o;

        if (failOnExistingSlice != slice.failOnExistingSlice) return false;
        if (failOnNonExistingSlice != slice.failOnNonExistingSlice) return false;
        if (expireTimeMin != slice.expireTimeMin) return false;
        if (renewExistingSliceIfNeeded != slice.renewExistingSliceIfNeeded) return false;
        if (failIfNoProject != slice.failIfNoProject) return false;
        if (sliceName != null ? !sliceName.equals(slice.sliceName) : slice.sliceName != null) return false;
        if (projectSource != slice.projectSource) return false;
        return project != null ? project.equals(slice.project) : slice.project == null;
    }

    @Override
    public int hashCode() {
        int result = sliceName != null ? sliceName.hashCode() : 0;
        result = 31 * result + (failOnExistingSlice ? 1 : 0);
        result = 31 * result + (failOnNonExistingSlice ? 1 : 0);
        result = 31 * result + (int) (expireTimeMin ^ (expireTimeMin >>> 32));
        result = 31 * result + (renewExistingSliceIfNeeded ? 1 : 0);
        result = 31 * result + projectSource.hashCode();
        result = 31 * result + (failIfNoProject ? 1 : 0);
        result = 31 * result + (project != null ? project.hashCode() : 0);
        return result;
    }

    public enum ProjectSource { NONE, AUTO, PROVIDED }
}
